from typing import Any

class BinaryView:
    file: Any
    end: int

    def read(self, addr: int, length: int) -> bytes: ...
    def add_auto_segment(
        self, start: int, length: int, data_offset: int = 0, data_length: int = 0, flags: Any = None
    ) -> Any: ...
    def add_user_section(
        self, name: str, start: int, length: int, semantics: Any = None
    ) -> Any: ...
    def parse_type_string(self, type_string: str) -> tuple[Any, ...]: ...
    def define_user_symbol(self, symbol: Any) -> None: ...
    def define_user_data_var(self, addr: int, var_type: Any) -> None: ...
    def define_user_type(self, name: str, type_obj: Any) -> None: ...
    def get_function_at(self, addr: int) -> Any | None: ...
    def create_user_function(self, addr: int) -> Any | None: ...
    def __init__(
        self, parent_view: BinaryView | None = None, file_metadata: Any = None
    ) -> None: ...
