from typing import Any

BranchType: Any
InstructionTextTokenType: Any
SegmentFlag: Any
SectionSemantics: Any
SymbolType: Any
Endianness: Any
FlagRole: Any
ImplicitRegisterExtend: Any
ActionType: Any
LowLevelILOperation: Any
LowLevelILFlagCondition: Any
